@@ -110,7 +110,7 @@ NvBool nvHsIoctlMoveCursor(
 {
     NVHsChannelEvoRec *pHsChannel;
 
-    if (apiHead > ARRAY_LEN(pDispEvo->pHsChannel)) {
+    if (apiHead >= ARRAY_LEN(pDispEvo->pHsChannel)) {
         return FALSE;
     }
 
@@ -169,6 +169,8 @@ static void HsIoctlSetCursorImage(
     NVHsChannelEvoRec *pHsChannel,
     NVSurfaceEvoRec *pSurfaceEvo)
 {
+    NVDevEvoPtr pDevEvo = pHsChannel->pDispEvo->pDevEvo;
+
     /*
      * Increment the refcnt of the new surface, and
      * decrement the refcnt of the old surface.
@@ -178,10 +180,10 @@ static void HsIoctlSetCursorImage(
      */
 
     HsChangeSurfaceFlipRefCount(
-        pSurfaceEvo, TRUE /* increase */);
+        pDevEvo, pSurfaceEvo, TRUE /* increase */);
 
     HsChangeSurfaceFlipRefCount(
-        pHsChannel->config.cursor.pSurfaceEvo, FALSE /* increase */);
+        pDevEvo, pHsChannel->config.cursor.pSurfaceEvo, FALSE /* increase */);
 
     pHsChannel->config.cursor.pSurfaceEvo = pSurfaceEvo;
 
@@ -204,7 +206,7 @@ NvBool nvHsIoctlSetCursorImage(
     NVHsChannelEvoRec *pHsChannel;
     NVSurfaceEvoRec *pSurfaceEvo = NULL;
 
-    if (apiHead > ARRAY_LEN(pDispEvo->pHsChannel)) {
+    if (apiHead >= ARRAY_LEN(pDispEvo->pHsChannel)) {
         return FALSE;
     }
 
