@@ -140,6 +140,8 @@ kgspAllocBootArgs_TU102
     pKernelGsp->pWprMeta = (GspFwWprMeta *)NvP64_VALUE(pVa);
     pKernelGsp->pWprMetaMappingPriv = pPriv;
 
+    portMemSet(pKernelGsp->pWprMeta, 0, sizeof(*pKernelGsp->pWprMeta));
+
     //
     // Setup libos arguments memory
     //
@@ -166,6 +168,8 @@ kgspAllocBootArgs_TU102
     pKernelGsp->pLibosInitArgumentsCached = (LibosMemoryRegionInitArgument *)NvP64_VALUE(pVa);
     pKernelGsp->pLibosInitArgumentsMappingPriv = pPriv;
 
+    portMemSet(pKernelGsp->pLibosInitArgumentsCached, 0, LIBOS_INIT_ARGUMENTS_SIZE);
+
     // Setup bootloader arguments memory.
     NV_ASSERT(sizeof(GSP_ARGUMENTS_CACHED) <= 0x1000);
 
@@ -190,6 +194,8 @@ kgspAllocBootArgs_TU102
     pKernelGsp->pGspArgumentsCached = (GSP_ARGUMENTS_CACHED *)NvP64_VALUE(pVa);
     pKernelGsp->pGspArgumentsMappingPriv = pPriv;
 
+    portMemSet(pKernelGsp->pGspArgumentsCached, 0, sizeof(*pKernelGsp->pGspArgumentsCached));
+
     return nvStatus;
 
 _kgspAllocBootArgs_exit_cleanup:
@@ -492,8 +498,9 @@ kgspGetGspRmBootUcodeStorage_TU102
  *   | GSP FW (non-WPR) HEAP    |
  *   ---------------------------- <- nonWprHeapOffset, gspFwRsvdStart
  *
- *  gspFwHeapOffset** contains the RM/Libos Heap. First 16 Mb are for Libos heap
- *  rest is for GSP-RM
+ *  gspFwHeapOffset** contains the entire WPR heap region, which can be subdivided
+ *  for various GSP FW components.
+ *
  * @param       pGpu          GPU object pointer
  * @param       pKernelGsp    KernelGsp object pointer
  * @param       pGspFw        Pointer to GSP-RM fw image.
@@ -523,8 +530,6 @@ kgspCalculateFbLayout_TU102
     NV_ASSERT_OR_RETURN(pKernelGsp->gspRmBootUcodeSize != 0, NV_ERR_INVALID_STATE);
     NV_ASSERT_OR_RETURN(pRiscvDesc != NULL, NV_ERR_INVALID_STATE);
 
-    portMemSet(pWprMeta, 0, sizeof *pWprMeta);
-
     NV_ASSERT_OK_OR_RETURN(kmemsysGetUsableFbSize_HAL(pGpu, pKernelMemorySystem, &pWprMeta->fbSize));
 
     //
@@ -558,8 +563,11 @@ kgspCalculateFbLayout_TU102
     else
         vbiosReservedOffset = pWprMeta->vgaWorkspaceOffset;
 
-    // End of WPR region (128KB aligned)
-    pWprMeta->gspFwWprEnd = NV_ALIGN_DOWN64(vbiosReservedOffset, 0x20000);
+    // Set the size of the GSP FW ahead of kgspGetWprEndMargin()
+    pWprMeta->sizeOfRadix3Elf = pGspFw->imageSize;
+
+    // End of WPR region (128KB aligned), shifted for any WPR end margin
+    pWprMeta->gspFwWprEnd = NV_ALIGN_DOWN64(vbiosReservedOffset - kgspGetWprEndMargin(pGpu, pKernelGsp), 0x20000);
 
     pWprMeta->frtsSize = kgspGetFrtsSize(pGpu, pKernelGsp);
     pWprMeta->frtsOffset = pWprMeta->gspFwWprEnd - pWprMeta->frtsSize;
@@ -568,9 +576,6 @@ kgspCalculateFbLayout_TU102
     pWprMeta->sizeOfBootloader = pKernelGsp->gspRmBootUcodeSize;
     pWprMeta->bootBinOffset = NV_ALIGN_DOWN64(pWprMeta->frtsOffset - pWprMeta->sizeOfBootloader, 0x1000);
 
-    // Compute GSP firmware image size
-    pWprMeta->sizeOfRadix3Elf = pGspFw->imageSize;
-
     //
     // Compute the start of the ELF.  Align to 64K to avoid issues with
     // inherent alignment constraints.
@@ -768,6 +773,22 @@ kgspResetHw_TU102
     return NV_OK;
 }
 
+static NvBool kgspCrashCatReportImpactsGspRm(CrashCatReport *pReport)
+{
+    NV_CRASHCAT_CONTAINMENT containment;
+
+    containment = crashcatReportSourceContainment_HAL(pReport);
+    switch (containment)
+    {
+       case NV_CRASHCAT_CONTAINMENT_RISCV_MODE_M:
+       case NV_CRASHCAT_CONTAINMENT_RISCV_HART:
+       case NV_CRASHCAT_CONTAINMENT_UNCONTAINED:
+           return NV_TRUE;
+       default:
+           return NV_FALSE;
+    }
+}
+
 NvBool
 kgspHealthCheck_TU102
 (
@@ -786,7 +807,8 @@ kgspHealthCheck_TU102
 
         while ((pReport = crashcatEngineGetNextCrashReport(pCrashCatEng)) != NULL)
         {
-            bHealthy = NV_FALSE;
+            if (kgspCrashCatReportImpactsGspRm(pReport))
+                bHealthy = NV_FALSE;
 
             NV_PRINTF(LEVEL_ERROR,
                 "****************************** GSP-CrashCat Report *******************************\n");
@@ -834,10 +856,19 @@ kgspHealthCheck_TU102
 exit_health_check:
     if (!bHealthy)
     {
+        NvBool bFirstFatal = !pKernelGsp->bFatalError;
+
         pKernelGsp->bFatalError = NV_TRUE;
 
         if (pKernelGsp->pRpc)
+        {
             kgspLogRpcDebugInfo(pGpu, pKernelGsp->pRpc, GSP_ERROR, pKernelGsp->bPollingForRpcResponse);
+        }
+
+        if (bFirstFatal)
+        {
+            kgspRcAndNotifyAllChannels(pGpu, pKernelGsp, GSP_ERROR, NV_TRUE);
+        }
 
         gpuCheckEccCounts_HAL(pGpu);
 
@@ -956,9 +987,6 @@ kgspIsWpr2Up_TU102
     return (wpr2HiVal != 0);
 }
 
-#define FWSECLIC_PROG_START_TIMEOUT     50000    // 50ms
-#define FWSECLIC_PROG_COMPLETE_TIMEOUT  2000000  // 2s
-
 NV_STATUS
 kgspWaitForGfwBootOk_TU102
 (
@@ -966,51 +994,16 @@ kgspWaitForGfwBootOk_TU102
     KernelGsp *pKernelGsp
 )
 {
-    NvU32 timeoutUs = FWSECLIC_PROG_START_TIMEOUT + FWSECLIC_PROG_COMPLETE_TIMEOUT;
-    RMTIMEOUT timeout;
     NV_STATUS status = NV_OK;
 
-    // Use the OS timer since the GPU timer is not ready yet
-    gpuSetTimeout(pGpu, gpuScaleTimeout(pGpu, timeoutUs), &timeout,
-                  GPU_TIMEOUT_FLAGS_OSTIMER);
-
-    while (status == NV_OK)
+    status = gpuWaitForGfwBootComplete_HAL(pGpu);
+    if (status != NV_OK)
     {
-        //
-        // Before reading the actual GFW_BOOT status register,
-        // we want to check that FWSEC has lowered its PLM first.
-        // If not then obviously it has not completed.
-        //
-        if (GPU_FLD_TEST_DRF_DEF(pGpu,
-                _PGC6,
-                _AON_SECURE_SCRATCH_GROUP_05_PRIV_LEVEL_MASK,
-                _READ_PROTECTION_LEVEL0,
-                _ENABLE)
-            )
-        {
-            if (GPU_FLD_TEST_DRF_DEF(pGpu,
-                    _PGC6,
-                    _AON_SECURE_SCRATCH_GROUP_05_0_GFW_BOOT,
-                    _PROGRESS,
-                    _COMPLETED)
-                )
-            {
-                return NV_OK;
-            }
-        }
-
-        status = gpuCheckTimeout(pGpu, &timeout);
+        NV_PRINTF(LEVEL_ERROR, "failed to wait for GFW boot complete: 0x%x VBIOS version %s\n",
+                  status, pKernelGsp->vbiosVersionStr);
+        NV_PRINTF(LEVEL_ERROR, "(the GPU may be in a bad state and may need to be reset)\n");
     }
 
-    // The wait failed if we reach here (as above loop returns upon success).
-    NV_PRINTF(LEVEL_ERROR, "failed to wait for GFW_BOOT: 0x%x (progress 0x%x, VBIOS version %s)\n",
-              status, GPU_REG_RD_DRF(pGpu,
-                        _PGC6,
-                        _AON_SECURE_SCRATCH_GROUP_05_0_GFW_BOOT,
-                        _PROGRESS),
-              pKernelGsp->vbiosVersionStr);
-    NV_PRINTF(LEVEL_ERROR, "(the GPU may be in a bad state and may need to be reset)\n");
-
     return status;
 }
 
